@@ -360,6 +360,13 @@ class BoboException(Exception):
         self.headers = headers or []
 
 def _scan_module(module_name):
+    # Scan a module for resources, and return a generator of resources
+    # in order. The order is defined by:
+    # - Order in source
+    # - Overridden order
+    # - route grouping
+    # In particular, wrt the last bullet, resources with the same route
+    # are grouped with the order of the first route.
     module = _import(module_name)
     bobo_response = getattr(module, 'bobo_response', None)
     if bobo_response is not None:
@@ -398,10 +405,13 @@ def _scan_module(module_name):
         yield bobo_response
 
 def _make_br_function_by_methods(route, by_method):
+    # Build a bobo_response function for one or more resources for a
+    # given route that define both route and methodd (standard
+    # resources, iow).
 
     route_data = _compile_route(route)
 
-    def bobo_response(request, path, method):
+    def bobo_response_function_by_method(request, path, method):
         handler = by_method.get(method)
         if handler is None:
             handler = by_method.get(None)
@@ -413,7 +423,7 @@ def _make_br_function_by_methods(route, by_method):
 
         return handler(request, path, method)
 
-    return bobo_response
+    return bobo_response_function_by_method
 
 def _uncomment(text, split=False):
     result = list(filter(None, (
@@ -577,6 +587,13 @@ class _cached_property(object):
 
 _ext_re = re.compile('/(\w+)').search
 class _Handler:
+    # Handlers wrap functions to provide the bobo resource interface.
+    #
+    # They handle requests by calling the undelying functions.
+    #
+    # An added complication is that handlers can be stacked, for
+    # example to use the same function for multiple routes. In
+    # addition, handlers can be called like the original function.
 
     partial = False
 
@@ -597,6 +614,7 @@ class _Handler:
         self.bobo_original = getattr(handler, 'bobo_original', handler)
         bobo_sub_find = getattr(handler, 'bobo_response', None)
         if bobo_sub_find is not None:
+            # We're stacked on another handler.
             self.bobo_sub_find = bobo_sub_find
 
         self.content_type = content_type
@@ -620,14 +638,14 @@ class _Handler:
         route_data = _compile_route(self.bobo_route, self.partial)
         methods = self.bobo_methods
         if methods is None:
-            return route_data
-
-        def match(request, path, method):
-            data = route_data(request, path)
-            if data is not None:
-                if method not in methods:
-                    raise MethodNotAllowed(methods)
-                return data
+            match = route_data
+        else:
+            def match(request, path, method):
+                data = route_data(request, path)
+                if data is not None:
+                    if method not in methods:
+                        raise MethodNotAllowed(methods)
+                    return data
 
         self.__dict__['match'] = match
         return match
@@ -1165,7 +1183,7 @@ def _compile_route(route, partial=False):
 
     if partial:
         match = re.compile(''.join(rpat)).match
-        def route_data(request, path, method=None):
+        def partial_route_data(request, path, method=None):
             m = match(path)
             if m is None:
                 return m
@@ -1174,6 +1192,8 @@ def _compile_route(route, partial=False):
                          if item[1] is not None),
                     path,
                     )
+
+        return partial_route_data
     else:
         match = re.compile(''.join(rpat)+'$').match
         def route_data(request, path, method=None):
@@ -1183,7 +1203,7 @@ def _compile_route(route, partial=False):
             return dict(item for item in six.iteritems(m.groupdict())
                         if item[1] is not None)
 
-    return route_data
+        return route_data
 
 def _make_bobo_handle(func, original, check, content_type):
 
@@ -1203,11 +1223,13 @@ def _make_bobo_handle(func, original, check, content_type):
 
     return handle
 
+_no_jget = {}.get
 def _make_caller(obj, paramsattr):
     spec = getargspec(obj)
     nargs = nrequired = len(spec.args)
     if spec.defaults:
         nrequired -= len(spec.defaults)
+    no_jget = _no_jget
 
     # XXX maybe handle f(..., **kw)?
 
@@ -1215,6 +1237,9 @@ def _make_caller(obj, paramsattr):
         request = pargs[-1]
         pargs = pargs[:-1] # () or (self, )
         params = getattr(request, paramsattr)
+        rget = route.get
+        pget = params.getall
+        jget = 0
         kw = {}
         for index in range(len(pargs), nargs):
             name = spec.args[index]
@@ -1222,15 +1247,23 @@ def _make_caller(obj, paramsattr):
                 kw[name] = request
                 continue
 
-            v = route.get(name)
+            v = rget(name)
             if v is None:
-                v = params.getall(name)
-                if not v:
-                    if index < nrequired:
-                        raise MissingFormVariable(name)
-                    continue
-                if len(v) == 1:
-                    v = v[0]
+                v = pget(name)
+                if v:
+                    if len(v) == 1:
+                        v = v[0]
+                else:
+                    if jget == 0:
+                        if request.content_type == 'application/json':
+                            jget = request.json.get
+                        else:
+                            jget = no_jget
+                    v = jget(name, request)
+                    if v is request:
+                        if index < nrequired:
+                            raise MissingFormVariable(name)
+                        continue
 
             kw[name] = v
 
@@ -1428,7 +1461,7 @@ def scan_class(class_):
 
         handlers.append(_make_br_method_for_name(name))
 
-    def bobo_response(self, request, path, method):
+    def instance_bobo_response(self, request, path, method):
         allowed = set()
         for handler in handlers:
             try:
@@ -1443,21 +1476,25 @@ def scan_class(class_):
 
     old = class_.__dict__.get('bobo_response')
     if isinstance(old, _subroute_class_method):
-        old.inst_func = bobo_response
+        old.inst_func = instance_bobo_response
     else:
-        class_.bobo_response = bobo_response
+        class_.bobo_response = instance_bobo_response
 
     return class_
 
 def _make_br_method_for_name(name):
-    return (lambda self, request, path, method:
-            getattr(self, name).bobo_response(request, path, method)
-            )
+    def custom_bobo_response_method(self, request, path, method):
+        # Handle a resource that just has bobo_response, but no metadata
+        return getattr(self, name).bobo_response(request, path, method)
+    return custom_bobo_response_method
 
 def _make_br_method_by_methods(route, methods):
+    # Make a combined bobo_response for one or more standard instance
+    # resource that have a common route and are distinguished by the
+    # methods they support.
     route_data = _compile_route(route)
 
-    def bobo_response(self, request, path, method):
+    def bobo_response_method_by_methods(self, request, path, method):
         name = methods.get(method)
         if name is None:
             name = methods.get(None)
@@ -1469,7 +1506,7 @@ def _make_br_method_by_methods(route, methods):
 
         return getattr(self, name).bobo_response(request, path, method)
 
-    return bobo_response
+    return bobo_response_method_by_methods
 
 class MissingFormVariable(Exception):
     def __init__(self, name):
